//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 26/06/02; 10:50:52

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "graphic.h"
#include "menulib.h"
#include "puzzle.h"

//--------------------------------------------------------------------------

extern TMenu*Menu;
enum {ME_MAIN,ME_PAUSE,ME_OPTIONS};

//--------------------------------------------------------------------------


char * strings[] = {"No","Yes"};

TGame*Game;

int ShowTileOnScreen;

char * Plane0;
char * Plane1;

//--------------------------------------------------------------------------

void RefreshScreen(void)
{
	int x,y;
	
	ClearGrayScreen2B(Plane0,Plane1);
	
	for (x=0;x<Game->SizeX;x++)
	for (y=0;y<Game->SizeY;y++)
	RefreshTile(x,y);
	
	FastCopyScreen(Plane0,GetPlane(0));
	FastCopyScreen(Plane1,GetPlane(1));
	
}
//--------------------------------------------------------------------------

void RefreshTile(int x, int y)
{
	char buffer[50];
	
	int posx=(x<<3)+(x<<2);
	int posy=(y<<3)+(y<<2);
	
	char*Screen0=ShowTileOnScreen?GetPlane(0):Plane0;
	char*Screen1=ShowTileOnScreen?GetPlane(1):Plane1;
	
	GraySprite16_AND(posx,posy,12,tile_Mask,tile_Mask,Screen0,Screen1);
	
	unsigned char TileData=Game->BoardMat[x][y];
	
	if (TileData!=NO_TILE)
	{
		GraySprite16_OR(posx,posy,12,tile_C0,tile_C1,Screen0,Screen1);
		sprintf(buffer,"%02d",TileData+1);
		DrawGrayStrExt2B(posx+1,posy+3,buffer,A_NORMAL,0,Screen0,Screen1);
	}
	else
	{
		GraySprite16_OR(posx,posy,12,no_tile_C0,no_tile_C1,Screen0,Screen1);
	}
	
	
}
//--------------------------------------------------------------------------


int MoveTile (int Move)
{
	
	TPoint NewPos=Game->BlankTilePos;
	
	switch (Move)
	{
		case TMOVE_L : NewPos.x++;break;
		case TMOVE_R : NewPos.x--;break;
		case TMOVE_U : NewPos.y++;break;
		case TMOVE_D : NewPos.y--;break;
	}
	
	if (NewPos.x>=0 && NewPos.x<Game->SizeX && NewPos.y>=0 && NewPos.y<Game->SizeY)
	{
		Game->BoardMat[Game->BlankTilePos.x][Game->BlankTilePos.y] = Game->BoardMat[NewPos.x][NewPos.y];
		Game->BoardMat[NewPos.x][NewPos.y] = NO_TILE;
		
		if (Game->show_moves)
		{
			ShowTileOnScreen=1;
			RefreshTile(Game->BlankTilePos.x,Game->BlankTilePos.y);
			RefreshTile(NewPos.x,NewPos.y);
			ShowTileOnScreen=0;
		}
		
		Game->BlankTilePos=NewPos;
		return 1;
	}
	else
	return 0;
	
}
//--------------------------------------------------------------------------

void MoveTileUser(int Move)
{
	
	if (MoveTile(Move))
	{
		
		Game->HistoryData[Game->moves] = (char)Move;
		Game->moves++;
	}
	Game->moves_max = Game->moves;
	
}
//--------------------------------------------------------------------------

void Shuffle(void)
{
	int a;
	int last_move=0;
	int ShuffleMove;
	
	for (a=0;a<SHUFFLE_MOVES;a++)
	{
		
		retry:
		
		ShuffleMove = random(4);
		
		if (a>0 && ShuffleMove==((last_move+2)&3))
		goto retry;
		
		if (!MoveTile(ShuffleMove))
		goto retry;
		
		last_move = ShuffleMove;
	}
	
}
//--------------------------------------------------------------------------

int IsSolved(void)
{
	int x,y;
	
	for (x=0;x<Game->SizeX;x++)
	for (y=0;y<Game->SizeY;y++)
	if (!(x==(Game->SizeX-1) && y==(Game->SizeY-1)))
	if (Game->BoardMat[x][y] != (Game->SizeX*y+x))
	{
		return 0;
	}
	
	return 1;
	
}
//--------------------------------------------------------------------------

void NewGame(void)
{
	
	int x,y;
	
	for (x=0;x<Game->SizeX;x++)
	for (y=0;y<Game->SizeY;y++)
	Game->BoardMat[x][y] = Game->SizeX*y+x;
	
	Game->BoardMat[Game->SizeX-1][Game->SizeY-1] = NO_TILE;
	Game->BlankTilePos.x = Game->SizeX-1;
	Game->BlankTilePos.y = Game->SizeY-1;
	
	Game->moves = 0;
	Game->moves_max = 0;
	
	Game->show_moves = 0;
	Shuffle();
	Game->show_moves = 1;
	
	RefreshScreen();
}
//--------------------------------------------------------------------------

void Undo(void)
{
	int MoveNum;
	
	if (Game->moves>0)
	{
		MoveNum = (int)Game->HistoryData[Game->moves-1];
		MoveTile((MoveNum+2)&3);
		Game->moves--;
		
	}
	
}
//--------------------------------------------------------------------------

void Redo(void)
{
	int MoveNum;
	
	if (Game->moves<Game->moves_max)
	{
		MoveNum = (int)Game->HistoryData[Game->moves];
		MoveTile(MoveNum);
		Game->moves++;
	}
	
}
//--------------------------------------------------------------------------

// Main Function
void _main(void)
{
	short key;
	char buffer[50];
	
	if (!(Plane0=malloc(2*LCD_SIZE)))
	goto end_prog;
	Plane1=Plane0+LCD_SIZE;
	
	TMenu MenuA;
	Menu = &MenuA;
	
	TGame GameA;
	Game=&GameA;
	
	randomize();
	
	Game->SizeX = 4;
	Game->SizeY = 4;
	
	ShowTileOnScreen=0;
	
	for (;;)
	{
		MenuNew2();
		MenuAddTitle("Puzzle");
		MenuAddItem("Play");
		MenuAddItem("Options");
		MenuAddItem("About");
		MenuAddItem("Quit");
		
		MenuDo(ME_MAIN);
		
		if (!Menu->ok) break;
		
		if (Menu->Index==1)
		{
			
			GrayOn();
			
			NewGame();
			RefreshScreen();
			
			for (;;)
			{
				
				
				if (IsSolved())
				{
					DrawGrayStrExt2B(STATS_LEFT,40,"You Won!",A_NORMAL,0,GetPlane(0),GetPlane(1));
					ngetchx();
					NewGame();
					RefreshScreen();
				}

				sprintf(buffer,"Moves : %04d",Game->moves);
				DrawGrayStrExt2B(STATS_LEFT,10,buffer,A_REPLACE,0,GetPlane(0),GetPlane(1));
				
				key = ngetchx();
				
				if (key==KEY_LEFT)
				MoveTileUser(TMOVE_L);
				else if (key==KEY_RIGHT)
				MoveTileUser(TMOVE_R);
				else if (key==KEY_UP)
				MoveTileUser(TMOVE_U);
				else if (key==KEY_DOWN)
				MoveTileUser(TMOVE_D);
				
				
				else if (key==KEY_BACKSPACE)
				Undo();
				else if (key==KEY_CLEAR)
				Redo();
				
				
				else if (key==KEY_F2)
				{
					NewGame();
					RefreshScreen();
				}
				else if (key==KEY_APPS)
				off();
				else if (key==KEY_ESC)
				{
					
					GrayOff();
					clrscr();
					printf("[ESC]   : Continue \n");
					printf("[ENTER] : Exit \n");
					while ((key = ngetchx())!=KEY_ENTER && key!=KEY_ESC);
					GrayOn();
					
					if (key==KEY_ENTER)
					break;
					
					RefreshScreen();
					
				}
				
			}
			
			GrayOff();
			
		}
		else if (Menu->Index==2)
		{
			for (;;)
			{
				
				MenuNew2();
				MenuAddTitle("Options");
				sprintf(buffer,"Width : %d",Game->SizeX);
				MenuAddItem(buffer);
				sprintf(buffer,"Height : %d",Game->SizeY);
				MenuAddItem(buffer);
				
				MenuDo(ME_OPTIONS);
				
				if (!Menu->ok) break;
				
				if (Menu->Index==1)
				{
					if (Game->SizeX<SIZEX_MAX)
					Game->SizeX++;
					else
					Game->SizeX = 3;
				}
				else if (Menu->Index==2)
				{
					if (Game->SizeY<SIZEY_MAX)
					Game->SizeY++;
					else
					Game->SizeY = 3;
				}
			}
		}
		else if (Menu->Index==3)
		{
			clrscr();
			FontSetSys(0);
			printf("About \n");
			printf("Puzzle 1.2 \n");
			printf("Programmed by Olivier Sangala \n");
			printf("http://olivier.sangala.free.fr \n");
			printf("olivier_sangala@hotmail.com \n");
			FontSetSys(1);
			ngetchx();
		}
		else if (Menu->Index==4)
		{
			break;
		}
		
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
	
}
